/* sysauth.c
**
** try to authenticate a user with the system password database
** (/etc/passwd, /etc/shadow)
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <pwd.h>

#ifndef NOSHADOW
#include <shadow.h>
#endif

#include "vxerror.h"
#include "dbif.h"
#include "tools.h"
#include "config.h"
#include "prefs.h"
#include "mrmacs.h"

dbif_rec *sysauth (dbif_rec *rec, char *em_addr)
{
    /* Was mu ich tun?
    ** Zuerst mu ich den Benutzernamen aus der E-Mailadresse extrahieren;
    ** Mit diesem mu ich dann in die /etc/passwd (bzw. /etc/shadow) gehen,
    ** um einen passenden Eintrag zu finden.
    ** Finde ich keinen passenden Eintrag, so mu ich einen leeren Benutzer-
    ** datensatz zurckgeben.
    ** Finde ich hingegen einen Datensatz, so mu ich herausfinden, wo mein
    ** E-Mailfolder eigentlich stehen soll; habe ich dies geklrt, dann mu
    ** ich einen Benutzerdatensatz knstlich erzeugen; dieser mu(!!!) ein
    ** gesetztes `nonstandard'-Flag enthalten!
    ** Bin ich damit fertig, dann ist der so erzeugte Datensatz das Ergebnis
    ** meiner Bemhungen.
    ** nderung vom 17.5.1999: Die `/etc/passwd'-berprfung kann nun auch
    ** im Fall von (standard) `domains' stattfinden. Dies ermglicht mir
    ** auch die Einrichtung von Benutzern in der `/etc/passwd' mit Foldern mit
    ** Indexdatei, fhrt aber dazu, da diese Benutzer sich theoretisch mehrere
    ** Mailboxen zulegen knnen - fr jede durch `VxMail' verwaltete virtuelle
    ** Domain genau eine. Daher sollte diese Eigenschaft mit GRSSTER Vorsicht
    ** benutzt werden.
    */
    struct passwd *pw;
    struct spwd *sp;
    char *user, *xp;
    dbif_rec *res = (dbif_rec *) 0;
    int rt = dbif_rectype(rec);
    if (rt == RT_NSUSER) { rt = RT_NSDOMAIN; }
    user = gdupl (__FILE__, __LINE__, em_addr);
    xp = strchr (user, '@'); if (xp) { *xp = '\0'; }
    xp = gconc (__FILE__, __LINE__, rec->folder, "/", user, NULL);
    /* I must switch back to uid 0 in order to be able to read the file
    ** `/etc/shadow'
    */
    setreuid (getuid(), getuid());

    ifnull (pw = getpwnam (user)) {
	gfree (xp);
	gfree (user);
	return res;
    }
#ifndef NOSHADOW
    if (!strcmp (pw->pw_passwd, "x") || !strcmp (pw->pw_passwd, "*")) {
	ifnull (sp = getspnam (user)) {
	    gfree (xp);
	    gfree (user);
	    return res;
	}
	res = dbif_mkrec (sp->sp_pwdp, xp,
			  (size_t) 0, (off_t) 0,
			  (rt == RT_NSDOMAIN ? RT_NSUSER : RT_USER));
	res->uid = pw->pw_uid; res->gid = pw->pw_gid;
	setgid (rt == RT_NSDOMAIN ? pw->pw_gid : prefs.vxgid);
	setreuid (getuid(), (rt == RT_NSDOMAIN ? pw->pw_uid : prefs.vxuid));
	gfree (xp);
	gfree (user);
	return res;
    }
#endif
    res = dbif_mkrec (pw->pw_passwd, xp,
		      (size_t) 0, (off_t) 0,
		      (rt == RT_NSDOMAIN ? RT_NSUSER : RT_USER));
    res->uid = pw->pw_uid; res->gid = pw->pw_gid;
    setgid (rt == RT_NSDOMAIN ? pw->pw_gid : prefs.vxgid);
    setreuid (getuid(), rt == RT_NSDOMAIN ? pw->pw_uid : prefs.vxuid);
    gfree (xp);
    gfree (user);
    return res;
}
