/* xlock.c
**
** file locking function which mimics (at most) the system V's `lockf' function
** but allows shared locks to be set, too.
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>

#include "xlock.h"

#ifndef FORCE_FCNTL
#ifdef __USE_BSD
int xLock (int fd, int cmd, off_t dummy)
{
    int action = ~XL_LKNB, flcmd;
    action &= cmd;
    switch (action) {
	case XL_TEST: errno = EACCES; return -1;
	case XL_UNLK: flcmd = LOCK_UN; break;
	case XL_LKSH: flcmd = LOCK_SH; break;
	case XL_LKEX: flcmd = LOCK_EX; break;
	default:      errno = EINVAL; return -1;
    }
    if (cmd & XL_LKNB) { flcmd |= LOCK_NB; }
    return flock (fd, flcmd);
}
#else
int xLock (int fd, int cmd, off_t size)
{
    int action = ~XL_LKNB, lfcmd;
    action &= cmd;
    switch (action) {
	case XL_TEST: lfcmd = F_TEST; break;
	case XL_UNLK: lfcmd = F_ULOCK; break;
	case XL_LKSH: errno = EACCES; return -1;
	case XL_LKEX: lfcmd = F_LOCK; break;
	default:      errno = EINVAL; return -1;
    }
    if (cmd & XL_LKNB) {
	if (lfcmd == F_LOCK) {
	    lfcmd = F_TLOCK;
	} else {
	    errno = EINVAL; return -1;
	}
    }
    return lockf (fd, lfcmd, size);
}
#endif
#else
int xLock (int fd, int cmd, off_t len)
{
    struct flock fl;
    int action;

    memset ((char *) &fl, '\0', sizeof (fl));

    fl.l_whence = SEEK_CUR;
    fl.l_start = 0;
    fl.l_len = len;

    action = ~XL_LKNB;
    action &= cmd;

    switch (action) {
	case XL_TEST:
	    if (fcntl (fd, F_GETLK, &fl) < 0) { return -1; }
	    if (fl.l_type == F_UNLCK || fl.l_pid == getpid ()) { return 0; }
	    errno = EACCES;
	    return -1;
	case XL_UNLK:
	    fl.l_type = F_UNLCK;
	    break;
	case XL_LKSH:
	    fl.l_type = F_RDLCK;
	    break;
	case XL_LKEX:
	    fl.l_type = F_WRLCK;
	    break;
	default:
	    errno = EINVAL;
	    return -1;
    }
    return fcntl (fd, ((cmd & XL_LKNB) ? F_SETLK : F_SETLKW), &fl);
}
#endif
